% Created 2010-May-05
% This file contains several examples for the structured H2 problem.
% In the TAC paper, we used the mass-spring example and the nine-vehicle
% formation example.

% -----------------------------------------------------------------------
% --------------------- mass-spring systems---------------------------
% % % % % problem setup see TAC paper;
% % % % % output measurements are position, velocity, and the average position,  
% % % % % h_i = (p_{i-1} + p_{i+1})/2;
% % % % % % H = toeplitz([0 0.5 0 0 0 0 ....])
% % % % % % C2 = [I Z; Z I; H Z];
% % % % % output is actually [p, v, pa],
% % % % % the structure gain F = [I I I]

% % problem data

clear
N = 20;
T = toeplitz([-2 1 zeros(1,N-2)]);
I = eye(N);
I2 = eye(2*N);
Z = zeros(N);
A = [Z, I; T, Z];
B = [Z; I];
B1 = B;
B2 = B;   
Q = I2;  
R = I;

% % different setup for outputs

% % C2 is identity

% % C2 = I2;
% % IS = [I I];

% %  C2 is a full-row rank matrix

% % C2 = [I I];
% % IS = I;

% %  C2 is a full-column rank matrix

H = toeplitz([0 0.5 zeros(1,N-2)]);
C2 = [I Z; Z I;  H Z];
IS = [I I I];

% ------------------ End of Mass-spring system -------------------------
% -----------------------------------------------------------------------


% -----------------------------------------------------------------------
% -----------------2D 9 vehicle formation example------------------------

% % 9 vehicles, each has 4 states, position, velocity at horizontal and
% % vertical directions

%     N = 9;
% 
%     I = eye(4*N);
%     
%     
% % % state-space representation of each vehicle
% 
% Aa = zeros(4); Aa(1,3)=1; Aa(2,4)=1; Za=zeros(4);
% Bb = zeros(4,2); Bb(3,1)=1; Bb(4,2)=1; Zb=zeros(4,2);
% 
% % % ss representation of the whole 9 vehicle system
% 
% A = zeros(N*size(Aa));
% B1 = zeros(N*size(Bb));
% B2 = zeros(N*size(Bb));
% [n,m] = size(Bb);
% 
% for i=1:N
%     A(n*(i-1)+1:n*i,n*(i-1)+1:n*i) = Aa;
%     B1(n*(i-1)+1:n*i,m*(i-1)+1:m*i) = Bb;    
%     B2(n*(i-1)+1:n*i,m*(i-1)+1:m*i) = Bb;        
% end
% 
% % Create the matrix Q; the index list describes all the nonzero entries of
% % Q; this list is obtained by hand calculation for this simple example;
% 
% idxlist = [1 2; 1 4; 1 5; 2 3; 2 4; 2 5; 2 6; 3 5; 3 6; 4 5; 
%             4 7; 4 8; 5 6; 5 7; 5 8; 5 9; 6 8; 6 9; 7 8; 8 9];
% 
% % Qr is the weight matrix for relative position errors        
%         
% Qr = zeros(4*N);
% for k=1:length(idxlist)
%     i = idxlist(k,1); j = idxlist(k,2);
%     Qr(4*(i-1)+1,4*(i-1)+1) = Qr(4*(i-1)+1,4*(i-1)+1)+1; 
%     Qr(4*(j-1)+1,4*(j-1)+1) = Qr(4*(j-1)+1,4*(j-1)+1)+1;
%     Qr(4*(i-1)+1,4*(j-1)+1) = -1; Qr(4*(j-1)+1,4*(i-1)+1) = -1;    
%     Qr(4*(i-1)+2,4*(i-1)+2) = Qr(4*(i-1)+2,4*(i-1)+2)+1; 
%     Qr(4*(j-1)+2,4*(j-1)+2) = Qr(4*(j-1)+2,4*(j-1)+2)+1;        
%     Qr(4*(i-1)+2,4*(j-1)+2) = -1; Qr(4*(j-1)+2,4*(i-1)+2) = -1;        
% end        
%         
% % Q is the weight matrix for both the relative and absolute position errors
% 
% Q = 10*Qr + diag(repmat([1 1 0 0 ]',N,1));
% 
% % % form the structural identity IS
% 
% IS = zeros(2*N,4*N);
% for i=1:N
%     IS(2*(i-1)+1:2*i,4*(i-1)+1:4*i) = ones(2,4);    
% end
% 
% % % control weight R and output matrix C2 are both identity
% 
% R = eye(2*N);
% C2 = eye(4*N);
% 
% % % ------------------Second part of 9-vehicle example------------------
% % % You need to have optimal dual variable V from the Augmented Lagrangian
% % % method to run the rest of the example
% 
% % % Step(1): divide the absolute values of V into m groups
% 
% m = 2;
% 
% % % the maximum absolute value of V
% 
% maxV = max(max(abs(V)));
% 
% % % the minimum absolute value of V that is also nonzero, note that V has
% % % zero entries at the block diagonal
% % % so it is necessary to add a tiny noise to set the minimum value
% 
% minV = min(min(abs(V))) + 1.e-30;
% dif = (maxV-minV)/m;
% 
% % % find the indexes for the dual variables in different groups
% 
% idx1 = find( minV        <= abs(V) & abs(V) <= minV+1*dif);
% idx2 = find( minV+1*dif  <  abs(V) & abs(V) <= minV+2*dif);
% 
% % % plotting the optimal dual variables in different groups
% 
% figure; 
% V1 = zeros(size(V));  V1(idx1)=1;
% V2 = zeros(size(V));  V2(idx2)=1;
% spy(V1,'xb',20);
% hold on
% spy(V2,'sr',15);
% 
% % % Step(2): change the structural identity for structure S1 or S2
% 
% % for structure S1
% 
%     IS = IS + V1;
%     
% % for structure S2
% 
%     IS = IS + V2;
    
% Then minimize the SH2 problem using the new structure chosen    

% ------------------------End of 9-vehicle example -----------------------
% -----------------------------------------------------------------------
